/******************************************************************************
*
* project name:    Kalaha
* file name:       Extern21.c
* initial date:    11.10.2004
* author:          mhubi@freesurf.ch
* description:     all to do with external files
* 
* last changes:		 12.10.2004 12:15
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//include the main-header-file
#include "main-h21.h"

//extern-variables used in this c-file
extern GAME game;
extern PLAYER player[2];


//prototype from the tigcc-tools-suite/ttmem
void Memory(void) {
	
	char string[100];
	
	//tigcclib: graph.h
	FontSetSys(F_4x6);
	
	//tigcclib: alloc.h (all functions that begin with "heap")
	HeapCompress(); // compress it first !!!
  //tigcclib: stdio.h , string.h
  sprintf(string, "avail=%lu bytes (maxblock=%lu)", HeapAvail(), HeapMax());
  //extgraph.h
  GrayDrawStr(32, 70, string, A_NORMAL);
  //own function
  WaitKey();
  //return;
	
}

//get the pointer to a file (from Travis Fischer's Excitebike 68k)
unsigned char *GetDataPtr(const char *Filename, short Offset) {
	
	//variables
	unsigned char *Location;
	SYM_ENTRY *SymPtr;
	
	//search the file
	//tigcclib: vat.h
	if ((SymPtr = SymFindPtr(SYMSTR(Filename), 0)) == NULL) {
		//error-handling if not found
		return NULL;
	}
	
	//grab the address
	//tigcclib: alloc.h
	if ((Location = (unsigned char*) HeapDeref(SymPtr->handle)) == NULL) {
		//error-handling
		return NULL;
	}
	
	//return the pointer
	return (Location + 2 + Offset);
	
}


//archives a file (from Travis Fischer's Excitebike 68k)
void Archive(const char *Filename) {
	
	//variables
	SYM_ENTRY *SymPtr = NULL;
	//tigcclib: vat.h
	HSym hsym = SymFind(SYMSTR(Filename));
	
	//tigcclib: vat.h
	SymPtr = DerefSym (hsym);
	
	//look if there could be garbage collection
	//tigcclib: flash.h
	if (EM_findEmptySlot(*GetDataPtr(Filename, -2)) == NULL) {
		return;
	}
	//archive it (if not already archive)
	if (!SymPtr->flags.bits.archived) {
		
		//tigcclib: vat.h
		EM_moveSymToExtMem(SYMSTR(Filename), HS_NULL);
		
	}
		
}


//unarchives a file (from Travis Fischer's Excitebike 68k)
void Unarchive(const char *Filename) {
	
	//variables
	SYM_ENTRY *SymPtr = NULL;
	//tigcclib: vat.h
	HSym hsym = SymFind(SYMSTR(Filename));
	
	//tigcclib: vat.h
	SymPtr = DerefSym (hsym);
	
	//if it's archive unarchive it
	if (SymPtr->flags.bits.archived) {
		
		//tigcclib: vat.h
		EM_moveSymFromExtMem(SYMSTR(Filename), HS_NULL);
		
	}
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void SaveOptions(void) {
	
	//variables
	FILE *f = NULL;
	SAVE_OPTIONS save_options;
	unsigned int success = TRUE;
	
	//put the variables into the corresponding structure-variable
	save_options.player[0] = player[0];
	save_options.player[1] = player[1];
	save_options.game = game;
	
	//unarchive it
	//own function
	Unarchive(optfile);
	
	//delete an old file
	//tigcclib: stdio.h
	unlink(optfile);
	
	//open the file for writing
	//tigcclib: stdio.h
	if ((f = fopen(optfile, "wb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//write structure to the file
		//tigcclib: stdio.h
		if (fwrite(&save_options, sizeof(SAVE_OPTIONS), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}
	
		//append the file id tag
		//tigcclib: stdio.h
		fputc(0, f);
		fputs("nfig", f);
		fputc(0, f);
		fputc(OTH_TAG, f);
		
		//close the file
		//tigcclib: stdio.h
		fclose(f);
		
	}
	
	//archive it
	//own function
	Archive(optfile);
	
	//handling the next-program-steps (success or not)
	if (success == TRUE) {
		
		//show success
		//own function
		MenuMainRoutine(M_OPTSAVED);
		m_choice = 0;
		
	}
	else {
		
		//show error
		//own function
		MenuMainRoutine(M_ERRORSAVE);
		m_choice = -1;
		
	}
	
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void LoadOptions(void) {
	
	//variables
	FILE *f = NULL;
	SAVE_OPTIONS save_options;
	unsigned int success = TRUE;
	
	//open file for reading
	//tigcclib: stdio.h
	if ((f = fopen(optfile, "rb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//read data from the file and put it into the save_option_structure
		//tigcclib: stdio.h
		if (fread(&save_options, sizeof(SAVE_OPTIONS), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}

		//close the file
		//tigcclib: stdio.h
		fclose(f);
		
	}
	
	//if success, put the values into the game-variables; handling next program-steps
	if (success == TRUE) {
		
		player[0] = save_options.player[0];
		player[1] = save_options.player[1];
		game = save_options.game;
		
		//show success
		//own function
		MenuMainRoutine(M_OPTLOADED);
		m_choice = 0;
		
	}
	else {
		
		//show error
		//own function
		MenuMainRoutine(M_ERRORLOAD);
		m_choice = -1;
		
	}
	
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void SaveGame(void) {
	
	//variables
	FILE *f = NULL;
	SAVE_GAME save_game;
	unsigned int success = TRUE;
	unsigned int loop1;
	
	//put the variables into the corresponding structure-variable
	save_game.player[0] = player[0];
	save_game.player[1] = player[1];
	save_game.game = game;
	for (loop1 = 0; loop1 <= 15; loop1++) {
		save_game.pot[loop1] = pot[F_REAL][loop1];
	}
	
	//unarchive the file
	//own function
	Unarchive(gamefile);
	
	//delete an old file
	//tigcclib: stdio.h
	unlink(gamefile);
	
	//open file for writing
	//tigcclib: stdio.h
	if ((f = fopen(gamefile, "wb")) == NULL) {
		//error-handling
		success = FALSE;
	}
	else {
		
		//write structure data to the file
		if (fwrite(&save_game, sizeof(SAVE_GAME), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}
	
		//append the file id tag
		//tigcclib: stdio.h
		fputc(0, f);
		fputs("save", f);
		fputc(0, f);
		fputc(OTH_TAG, f);
		
		//close the file
		//tigcclib: stdio.h
		fclose(f);
		
	}
	
	//archive the file
	//own function
	Archive(gamefile);
	
	//handling the next-program-steps (success or not)
	if (success == TRUE) {
		
		//show success
		//own function
		MenuMainRoutine(M_GAMESAVED);
		m_choice = 0;
		
	}
	else {
		
		//show error
		//own function
		MenuMainRoutine(M_ERRORSAVE);
		m_choice = -1;
		
	}
	
	//set status
	game.status = QUIT;
	
}


//base taken from www.technoplaza.net, c-programming, file i/o
void LoadGame(void) {
	
	//variables
	FILE *f = NULL;
	SAVE_GAME save_game;
	unsigned int success = TRUE;
	unsigned int loop1;
	
	//open file for reading
	//tigcclib: stdio.h
	if ((f = fopen(gamefile, "rb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//read data from file
		//tigcclib: stdio.h
		if (fread(&save_game, sizeof(SAVE_GAME), 1, f) != 1) {
			
			//error-handling
			success = FALSE;
			
		}

		//close the file
		//tigcclib: stdio.h
		fclose(f);
		
	}
	
	//if success, put the values into the game-variables; handling next program-steps
	if (success == TRUE) {
		
		player[0] = save_game.player[0];
		player[1] = save_game.player[1];
		game = save_game.game;
		for (loop1 = 0; loop1 <= 15; loop1++) {
			pot[F_REAL][loop1] = save_game.pot[loop1];
		}
		
		//show success
		//own function
		MenuMainRoutine(M_GAMELOADED);
		m_choice = 2;
		
	}
	else {
		
		//show error
		//own function
		MenuMainRoutine(M_ERRORLOAD);
		m_choice = -1;
		
	}
	
	
}


//saves the hiscore table to the hiscore file
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int SaveHiscores(SCORE *hiscores) {
	
	//variables
	unsigned int success = TRUE;
	FILE *f;
	
	//unarchive file
	//own function
	Unarchive(scorefile);
	
	//delete any old scorefile
	//tigcclib: stdio.h
	unlink(scorefile);

	//open the hiscore file
	//tigcclib: stdio.h
	if ((f = fopen(scorefile, "wb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//write the hiscore table to disk - check for errors
		//tigcclib: stdio.h
		if (fwrite(hiscores, sizeof(SCORE), MAX_HISCORES, f) != MAX_HISCORES) {
			
			//error-handling
			success = FALSE;
			
		}

		//write the file tag
		//tigcclib: stdio.h
		fputc(0, f);
		fputs("scor", f);
		fputc(0, f);
		fputc(OTH_TAG, f);
		
		//close the file
		//tigcclib: stdio.h
		fclose(f);
		
	}
	
	//archive the file
	//own function
	Archive(scorefile);
	
	return success;
	
}


//loads the hiscores from file into the hiscore table
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int LoadHiscores(SCORE *hiscores) {
	
	//variables
	unsigned int success = TRUE;
	FILE *f;

	//open file for reading and check for errors
	//tigcclib: stdio.h
	if ((f = fopen(scorefile, "rb")) == NULL) {
		
		//error-handling
		success = FALSE;
		
	}
	else {
		
		//read the hiscore table and check for errors
		//tigcclib: stdio.h
		if (fread(hiscores, sizeof(SCORE), MAX_HISCORES, f) != MAX_HISCORES) {
			
			//error-handling
			success = FALSE;
			
		}
		
		//close file
		//tigcclib: stdio.h
		fclose(f);
		
	}

	return success;
	
}


//create a blank hiscore table and write it to file
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int CreateHiscoreFile(void) {
	
	//variables
	unsigned int loop1;
	SCORE board[MAX_HISCORES];

	//set all the entries in the table to default (defined in data##.h)
	for (loop1 = 0; loop1 < MAX_HISCORES; loop1++) {
		
		//copy the score
		board[loop1].score = hisc_score[loop1];
		
		//copy the name
		strcpy(board[loop1].name, hisc_name[loop1]);
		
	}
	
	//save it and return this value
	//changed function from www.technoplaza.net
	return (SaveHiscores(board));
	
}

//checks whether a hiscore file already exists
//prototype from www.technoplaza.net und C-Programming/Graduate Review (Nibbler-game)
unsigned int NeedHiscoreFile(void) {
	
	//variables
	unsigned int needed = FALSE;
	//tigcclib: stdio.h
	FILE *f = fopen(scorefile, "rb");
	
	//if we cannot open the file, we must announce it to the previous routine
	if (f == NULL) {
		needed = TRUE;
	}
	else {
		
		//close the file
		//changed function from www.technoplaza.net
		fclose(f);
		
	}

	return needed;
	
}


//=============================================================================
// file history
//=============================================================================
//
//	11.10.2004	added getdataptr(), archive(), unarchive(), load/saveoptions()
//							load/savegame()
//	12.10.2004	added load/savehiscore(), needhiscorefile(), createhiscorefile()
