/******************************************************************************
*
* project name:		 Kalaha 
* file name:			 Game-Routines21.c
* initial date:		 15.09.2004
* author:					 mhubi@freesurf.ch
* description:		 game-(specific)-routines
* 
* last changes:		 13.10.2004 17:00
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//include main-header-file
#include "main-h21.h"

//extern-variables used in this c-file
extern unsigned int pot[8][15];
extern GAME game;
extern PLAYER player[2];


//the game-loop
void GameLoop(void) {
	
	//variables
	int key = 0;
	unsigned int loop1;
	
	//WHILE (THE GAME ISN'T FINISHED)
	while (game.status == NORMAL) {
		
		//IF (A HUMAN PLAYER MUST CHOOSE A MOVE)
		if (player[game.cur_pl].type == P_HUMAN) {
			
			//draw the arrow
			//own function
			DrawArrow();
			
			//WAIT UNTIL HE HAS CHOSEN A MOVE;
			//a second loop, because else if the player makes an input, every time the whole loop would be executed
			while (game.status == NORMAL) {
				
				//wait until the user presses a relevant key
				//own function
				while (!(key = KeyPressed())) {
					
					//set the calc in low-power-mode (wakes cpu only up for grayscale, keypress, amstimer, clock)
					//tigcclib: peekpoke.h
					pokeIO(0x600005, 0b10111);
					
				}
				
				//fastquit possibility
				if (key == K_F2) {
					
					//set the status value
					game.status = FASTQUIT;
					
					//call the endblock of this program
					LeadOut();
					
				}
				
				
				//choose exit
				else if (key == K_ESC) {
					
					//call the exit-menu and execute the chosen function
					//own function
					MenuMainRoutine(M_EXIT);
					(*menu_func[menu[M_EXIT].str_start + m_choice]) ();
					
					
				}
				//increase the contrast
				else if (key == K_PLUS) {
					
					//tigcclib: system.h
					OSContrastUp();
					
					//own function
					Wait();
					
				}
				//decrease the contrast
				else if (key == K_MINUS) {
					
					//tigcclib: system.h
					OSContrastDn();
					
					//own function
					Wait();
					
				}
				
				//choose the arrow position
				else if (key == K_RIGHT || key == K_LEFT) {
					
					//call the specific function
					//own function
					MoveArrow(key);
					
					//own function
					Wait();
				}
				
				//executing move
				else if (key == K_ENTER && pot[F_REAL][game.cur_pos] != 0) {
					
					//change the status, for executing it in the global block
					game.status = EXECMOVE;
				}
				
			}
			
		}
		
		//ELSE
		else {
			
			//CALL THE "ARTIFICAL INTELLIGENCE"-FUNCTION;
			//CHOOSE THIS POT FOR EXECUTING THE MOVE;
			//own function
			DynamicAi();
			
			//draw the chosen position
			//own function
			DrawArrow();
			
			//wait until the loop is ended: waited enough time or user hit enter or fastquit
			for (loop1 = 0; loop1 <= game.speed*500 &&  game.status == NORMAL; loop1++) {
				
				//slow down cpu
				//tigcclib: peekpoke.h
				pokeIO(0x600005, 0b10111);
				
				//check the keyboard
				//own function
				key = KeyPressed();
				
				//set the possible status
				if (key == K_ENTER) {
					game.status = EXECMOVE;
				}
				else if (key == K_F2) {
					
					//set the status value
					game.status = FASTQUIT;
					
					//call the endblock of this program
					LeadOut();
					
				}
				
			}
			
			//change the status, for executing it in the global block
			game.status = EXECMOVE;
			
		}

		//IF (THERE'S A MOVE TO EXECUTE)
		//general executing block for any player-type
		if (game.status == EXECMOVE) {
			
			//set back the game.status
			game.status = NORMAL;
			
			//save the arrow-position into the player-own-variable
			player[game.cur_pl].arr_pos = game.cur_pos;
			
			//CALL THE FUNCTION THAT EXECUTES THE MOVE;
			if (game.rule == 1) {
				
				//execute the real-move
				//own function
				ExecMove1(game.cur_pos);
				
			}
			else {
				
				//own function
				ExecMove2(game.cur_pos);
				
			}
			
			//delete the arrow
			//own function
			DelArrow();
			
			//SET THE VALUES FOR THE NEXT ROUND (EG. NEXT PLAYER);
			//own function
			NextRound();
			
			//own function
			Wait();
			
		}
		
	}
	
	
}


//initialise the game variables
void InitGame(void) {
	
	//set the status to normal
	game.status = NORMAL;
	
	//set all the variables to default
	game.speed = 2;
	game.stones = 6;
	game.moves = 0;
	game.rule = 1;
	
}


//set the pots to the beginning values in the real field
void InitPots(void) {
	
	//variables
	unsigned int loop1;
	
	//for-loop for all pots
	for (loop1 = 1; loop1 <= 14; loop1++) {
		
		// if it's pot 7 or 14 --> no content
		if (loop1%7 == 0) {
			pot[F_REAL][loop1] = 0;
		}
		
		// all others have six stones
		else {
			pot[F_REAL][loop1] = game.stones;
		}
		
		
	}
	
	
}


//distributes the stones of the chosen pot among the next pots, rules #1
void ExecMove1(unsigned int position) {
	
	//variables
	unsigned int potnum, stones, loop1;
	
	//increase the gamemove by one
	game.moves++;
	
	//clear the chosen pot (number of stones in a variable)
	stones = pot[F_REAL][position];
	pot[F_REAL][position] = 0;
	
	//do it also graphically
	//own function
	ClearPot(position);
	
	//on each following pot a stone while there's one left
	for (potnum = position + 1; stones > 0 && game.status != FASTQUIT; stones--, potnum++) {
		
		//set back so there's an array-circle
		if (potnum > 14) {
			potnum = 1;
		}
		
		//increase the content of a pot
		pot[F_REAL][potnum]++;
		
		//do it also graphically and wait between the stones
		//own function
		DrawStones(potnum);
		
		//slow down the whole for-loop, so you can follow
		for (loop1 = 0; loop1 <= game.speed*50 && stones > 1; loop1++) {
			
			//sets the calc in lower mode --> less energy-consumption
			//wait until the grayscale, the clock or the ams-timer wakes up the cpu
			//tigcclib: peekpoke.h
			pokeIO(0x600005, 0b10101);
			
			//fastquit possibility
			//own function
			if (_keytest(RR_F2)) {
				
				//set the status value
				game.status = FASTQUIT;
				
				//call the endblock of this program
				LeadOut();
								
			}
			
		}
		
		
	}
		
	//checking the special-rules
	//own function
	CheckRules1(potnum - 1);
	
	//check if the game has a winner or is draw
	//own function
	CheckWinner();
	
}


//distributes the stones of the chosen pot among the next pots, rules #2
void ExecMove2(unsigned int position) {
	
	//variables
	unsigned int potnum, stones, loop1;
	
	//increase the gamemove by one
	game.moves++;
	
	//clear the chosen pot (number of stones in a variable)
	stones = pot[F_REAL][position];
	pot[F_REAL][position] = 0;
	
	//do it also graphically
	//own function
	ClearPot(position);
	
	//on each following pot a stone while there's one left
	for (potnum = position + 1; stones > 0 && game.status != FASTQUIT; stones--, potnum++) {
		
		//set back so there's an array-circle
		if (potnum > 14) {
			potnum = 1;
		}
		else if (potnum == 14 && position < 7) {
			potnum = 1;
		}
		else if (potnum == 7 && position > 7) {
			potnum = 8;
		}
		
		//increase the content of a pot
		pot[F_REAL][potnum]++;
		
		//do it also graphically and wait between the stones
		//own function
		DrawStones(potnum);
		
		//slow down the whole for-loop, so you can follow
		for (loop1 = 0; loop1 <= game.speed*50 && stones > 1; loop1++) {
			
			//sets the calc in lower mode --> less energy-consumption
			//wait until the grayscale, the clock or the ams-timer wakes up the cpu
			//tigcclib: peekpoke.h
			pokeIO(0x600005, 0b10101);
			
			//fastquit possibility
			//own function
			if (_keytest(RR_F2)) {
				
				//set the status value
				game.status = FASTQUIT;
				
				//call the endblock of this program
				LeadOut();
				
			}
			
		}
		
		
	}
		
	//checking the special-rules
	//own function
	CheckRules2(potnum - 1);
	
	//check if the game has a winner or is draw
	//own function
	CheckWinner();
	
}


//checks if in this case the rules must be respected, rules #1
void CheckRules1(unsigned int potnum) {
	
	//checks if the last stone ends in the own big pot
	if (potnum/(game.cur_pl + 1) == 7) {
		
		//increases the player by two so he's also the next player
		game.cur_pl += 2;
		
	}
	
	//if the first rule doesn't have to be respected check the second rule
	//check if the last stone doesn't end in a big pot, ends on the own side and there's only one stone in
	//this pot
	
	else if (potnum%7 != 0 && potnum > game.cur_pl*7 && potnum < (game.cur_pl + 1)*7 && pot[F_REAL][potnum] == 1) {
		
		//the big pot gets the stones of the opposite pot
		pot[F_REAL][7 + game.cur_pl*7] += pot[F_REAL][14 - potnum];
		pot[F_REAL][14 - potnum] = 0;
		
		//make it also graphically
		//own functions
		ClearPot(14 - potnum);
		DrawStones(7 + game.cur_pl*7);
		
	}
	
	
}


//checks if in this case the rules must be respected, rules #2
void CheckRules2(unsigned int potnum) {
	
	//checks if the last stone ends in the own big pot
	if (potnum/(game.cur_pl + 1) == 7) {
		
		//increases the player by two so he's also the next player
		game.cur_pl += 2;
		
	}
	
	//if the first rule doesn't have to be respected check the second rule
	//check if the last stone doesn't end in a big pot, ends on the own side and there's only one stone in
	//this pot
	
	else if (potnum%7 != 0 && potnum > game.cur_pl*7 && potnum < (game.cur_pl + 1)*7 && pot[F_REAL][potnum] == 1 && pot[F_REAL][14-potnum] != 0) {
		
		//the big pot gets the stones of the opposite pot
		pot[F_REAL][7 + game.cur_pl*7] += pot[F_REAL][14 - potnum] + 1;
		pot[F_REAL][14 - potnum] = 0;
		pot[F_REAL][potnum] = 0;
		
		//make it also graphically
		//own functions
		ClearPot(14 - potnum);
		ClearPot(potnum);
		DrawStones(7 + game.cur_pl*7);
		
	}
	
	
}


//check if the game ends
void CheckWinner(void) {
	
	unsigned int loop1, sum;
	
	//if one condition is true you don't have to check other cases --> else
	//check if there are more than half of the stones in a big pot and set than the specific status
	if (pot[F_REAL][7] > game.stones*6) {
		game.status = P0_WON;
	}
	
	else if (pot[F_REAL][14] > game.stones*6) {
		game.status = P1_WON;
	}
	
	// check if there's a draw game, must come before the next, because in this case there aren't any
	// stones in the little pots --> "random" winner
	else if (pot[F_REAL][7] == game.stones*6 && pot[F_REAL][14] == game.stones*6) {
		game.status = DRAWGAME;
	}
	
	else {
		//look if the next player doesn't have any stones --> the other one is the winner
		sum = 0;
		
		//sum up all the stones in the little pots of the next player, but exits if there is still a stone
		for (loop1 = 1; loop1 <= 6; loop1++) {
			sum += pot[F_REAL][loop1 + next_player[game.cur_pl]*7];
		}
		
		//if there aren't any stones on the next-player's side, set the winner-value and give the rest of
		//the stones to the winner
		if (sum == 0) {
			
			if (game.cur_pl == 0 || game.cur_pl == 3) {
				game.status = P0_WON;
				pot[F_REAL][7] =  game.stones*12 - pot[F_REAL][14] + 1;
			}
			
			else if (game.cur_pl == 1 || game.cur_pl == 2) {
				game.status = P1_WON;
				pot[F_REAL][14] = game.stones*12 - pot[F_REAL][7] + 1;
			}
			
		}
		
	}
	
	
}


//sets the value for the next round
void NextRound(void) {
	
	//set values for the next round
	game.cur_pl = next_player[game.cur_pl];
	game.cur_pos = player[game.cur_pl].arr_pos;
	
}


//=============================================================================
// file history
//=============================================================================
//
//  8.7.2004 		initpots()
//	9.7.2004 		execmove(); deleted #includes
//	12.7.2004		checkrules(); new externs --> many changes of variable, less parameter
//							by functions
//	24.7.2004		added checkwinner();
//	25.7.2004 	corrected checkwinner()
//	31.8.2004 	changed completely InitPlayer(), because of the new game-situation
//	 2.8.2004 	added the 2nd index of pot;
//							added new parameters, because of the integration of the ai,
//							in all routines added if (field == F_REAL) before do it also graphically
//	10.8.2004 	changed checkrules() because of the address-error --> change of the
//							if-loops
//  13.8.2004		added the license
//	15.9.2004		added new summation-routine in checkwinner; deleted all the field-parameter
//							because for the ai, i have specific routines for speed-reasons
//	9.10.2004		added the fastquit's, deleted initplayer, changed initgame
//	11.10.2004	changed checkwinner (-->variable stone-number)
//	12.10.2004	added game-loop
//	13.10.2004	fixed bug in checkwinner
