/******************************************************************************
*
* project name:    Kalaha
* file name:       Gui21.c
* initial date:    15.09.2004
* author:          mhubi@freesurf.ch
* description:     graphical user interface
* 
* last changes:		 16.9.2004 17:00
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//include the main-header-file
#include "main-h21.h"

//extern-variables used in this c-file
extern GAME game;
extern PLAYER player[2];
extern int m_choice;

//moves the arrow
void MoveArrow(int key) {
	
	//first: delete the arrow
	//own function
	DelArrow();
	
	//set the new (or old) value
	if (key == K_RIGHT && game.cur_pos != 6 + game.cur_pl*2) {
		game.cur_pos += 1 - game.cur_pl*2;
	}
	if (key == K_LEFT && game.cur_pos != 1 + game.cur_pl*12) {
		game.cur_pos -= 1 - game.cur_pl*2;
	}
	
	//finally draw the arrow on the actual position
	//own function
	DrawArrow();
	
}

//this function controls all menus (only the number of the menu is necessary)
void MenuMainRoutine(unsigned int m_number) {
	
	//variables
	unsigned int arr_pos;
	int key;
	
	key = 0;
	m_choice = 0;
	arr_pos = 1;
	
	//call the draw-function
	//own function
	MenuDrawWindow(&menu[m_number]);
	
	//call the text-function
	//own function
	MenuDrawText(&menu[m_number]);
	
	//call the arrow-function
	//own function
	MenuDrawArrow(&menu[m_number], arr_pos);
	
	//call the choice-function
	//own function
	MenuMakeChoice(&menu[m_number], &arr_pos);
	
	//delete the screen (own method)
	//own function
	DelScreen();
	
}


//draws the menu-frame
void MenuDrawWindow(MENU *menu) {
	
	//draw the filled, light gray rectangle-frame
	//extgraph.h
	GrayFastFillRect_R(LPLANE, DPLANE, menu->x, menu->y, menu->x + menu->width, menu->y + menu->height, COLOR_LIGHTGRAY);
	GrayFastFillRect_R(LPLANE, DPLANE, menu->x + 3, menu->y + 3, menu->x + menu->width - 3, menu->y + menu->height - 3, COLOR_WHITE);
	
	//draw the black rectangle-frames
	//extgraph.h
	GrayFastOutlineRect_R(LPLANE, DPLANE, menu->x, menu->y, menu->x + 3, menu->y + menu->height, COLOR_BLACK);
	GrayFastOutlineRect_R(LPLANE, DPLANE, menu->x + menu->width - 3, menu->y, menu->x + menu->width, menu->y + menu->height, COLOR_BLACK);
	GrayFastOutlineRect_R(LPLANE, DPLANE, menu->x, menu->y, menu->x + menu->width, menu->y + 3, COLOR_BLACK);
	GrayFastOutlineRect_R(LPLANE, DPLANE, menu->x, menu->y + menu->height - 3, menu->x + menu->width, menu->y + menu->height, COLOR_BLACK);
	
	//reduce the corner of the menu to light gray
	//own function
	DelCorners(DPLANE, menu->x, menu->y, menu->x + menu->width, menu->y + menu->height);
	
}


//draws all the items of the menu
void MenuDrawText(MENU *menu) {
	
	unsigned int loop1;
	
	//set the big font
	//tigcclib: graph.h
	FontSetSys(F_6x8);
	
	//draw the title black
	//extgraph.h
	GrayDrawStrExt2B(0, menu->y + 6, menu_text[menu->str_start], A_NORMAL | A_CENTERED, F_8x10, LPLANE, DPLANE);
	
	//set the middle font
	//tigcclib: graph.h
	FontSetSys(F_6x8);
	
	//set the dark gray plane --> the following text is in dark gray
	//tigcclib: gray.h
	GraySetAMSPlane(1);
	
	//loop through all the menu points
	for (loop1 = 0; loop1 < menu->menu_points; loop1++) {
		
		//draw each menu-point on its position
		//tigcclib: graph.h
		DrawStr(menu->x + 17, menu->y + 19 + loop1*9, menu_text[menu->str_start + loop1 + 1], A_NORMAL);
		
	}
	
	
}


//draw the menu-arrow at the current position
void MenuDrawArrow(MENU *menu, unsigned int arr_pos) {
	
	//draw the ">"
	//extgraph.h
	GrayDrawStr(menu->x + 8, menu->y + 10 + arr_pos*9, (char *)">", A_NORMAL);
		
}


//deletes the arrow
void MenuDelArrow(MENU *menu, unsigned int arr_pos) {
	
	//delete it at the position by a <SPACE>
	//extgraph.h
	GrayDrawStr(menu->x + 8, menu->y + 10 + arr_pos*9, (char *)" ", A_SHADED);
	
}


//returns the chosen menu-point
void MenuMakeChoice(MENU *menu, unsigned int *arr_pos) {
	
	int key;
	
	//loops until the user's made a choice
	while (m_choice == 0) {
		
		//wait for the input
		//own function
		while (!(key = KeyPressed())) {
			
			//put the cpu in sleep
			//tigcclib: peekpoke.h
			pokeIO(0x600005, 0b10111);
			
		}
		
		//own function
		Wait();
		
		//fastquit-possibility
		if (key == K_F2) {
			
			//set the status value
			game.status = FASTQUIT;
			
			//go to the end block
			LeadOut();
						
		}
		
		//if the user choose up and he hasn't already reached the top
		if (key == K_UP && *arr_pos > 1) {
			
			//delete the arrow
			//own function
			MenuDelArrow(menu, *arr_pos);
			
			//decrease the position-value
			--*arr_pos;
			
			//draw the arrow at the new position
			//own function
			MenuDrawArrow(menu, *arr_pos);
			
		}
		
		//if the user choose down and he hasn't already reached the bottom
		if (key == K_DOWN && *arr_pos < menu->menu_points) {
			
			//delete the arrow
			//own function
			MenuDelArrow(menu, *arr_pos);
			
			//increase the position-value
			++*arr_pos;
			
			//draw the arrow at the new position
			//own function
			MenuDrawArrow(menu, *arr_pos);
			
		}
		
		//if the user wants to choose the current menu-point
		if (key == K_ENTER) {
			
			//set the menu-choice
			m_choice = *arr_pos;
			
		}
		
	}
	
	
}


//displays error, if the menu returns an impossible value
void Error(void) {
	
	//set the status value
	game.status = ERROR;
	
	//go to the endblock of the program
	LeadOut();
	
}


//call if return chosen
void Nothing(void) {
	
	//own function
	Wait();
	
}


//main routine to init a new game
void NewGame(void) {
	
	unsigned int loop1;
	
	//ask all the things about the two players
	for (loop1 = 0; loop1 <= 1; loop1++) {
		
		//display which player should choose
		//own function
		MenuMainRoutine(M_PL1 + loop1);
		
		//set the arrow-position
		player[loop1].arr_pos = 1 + loop1*7;
		
		//player-type menu
		//own function
		MenuMainRoutine(M_PLTYPE);
		
		//set player-type
		//human
		if (m_choice == 1) {
			player[loop1].type = P_HUMAN;
		}
		//computer (choose the strength too)
		else {
			player[loop1].type = P_CALC;
			
			//call the menu and set the value of the ai-strength
			//own function
			MenuMainRoutine(M_AI);
			player[loop1].ai_strength = m_choice;
			
		}
		
	}
	
	//call the menu for the beginning player and call Firstplayer()
	//own function
	MenuMainRoutine(M_1STPLAYER);
	(*menu_func[menu[M_1STPLAYER].str_start + m_choice]) ();
	
}


//if the user want to quit
void End(void) {
	game.status = QUIT;
}


//set the value of the beginning user
void FirstPlayer(void) {
	
	//player 1
	if (m_choice == 1) {
		game.cur_pl = 0;
	}
	//player 2
	else if (m_choice == 2) {
		game.cur_pl = 1;
	}
	//random
	else {
		game.cur_pl = random(2);
	}
	
	//set to neutral value
	m_choice = 0;
	
	//set the arrow-position
	game.cur_pos = player[game.cur_pl].arr_pos;
	
}




//function for menu-options call and the following menus
void MenuOptions(void) {
	
	//own function
	MenuMainRoutine(M_OPTIONS);
	(*menu_func[menu[M_OPTIONS].str_start + m_choice]) ();
	
}


//set the game-speed
void MenuSpeed(void) {
	
	//call the menu
	//own function
	MenuMainRoutine(M_SPEED);
	
	//set the value (except if it's return)
	if (m_choice != 4) {
		game.speed = 4 - m_choice;
	}
	
	//set it to a neutral value
	m_choice = 0;
	
}


//set the number of stones
void MenuStones(void) {
	
	//call the menu
	//own function
	MenuMainRoutine(M_STONES);
	
	//set the value (except if it's return)
	if (m_choice != 7) {
		game.stones = m_choice;
	}
	
	//set it to a neutral value
	m_choice = 0;
	
}


//set the rules
void MenuRule(void) {
	
	//own function
	MenuMainRoutine(M_RULE);
	
	//set the value (except if it's return)
	if (m_choice != 3) {
		game.rule = m_choice;
	}
	
	//set it to a neutral value
	m_choice = 0;
	
}


//help-menu-handler-function
void Help(void) {
	
	//call the menu and execute wished function
	//own function
	MenuMainRoutine(M_HELP);
	(*menu_func[menu[M_HELP].str_start + m_choice]) ();
	//return a neutral value
	m_choice = 0;
	
}


//display help-pages
void HelpPages(void) {
	
	//first entry chosen
	if (m_choice == 1) {
		
		//show the key-config in a window
		//own functions
		MenuDrawWindow(&menu[M_KEYCONFIG]);
		MenuDrawText(&menu[M_KEYCONFIG]);
		//no cursor!!
		
	}
	else {
		
		//show the about in a window
		//own functions
		MenuDrawWindow(&menu[M_ABOUT]);
		MenuDrawText(&menu[M_ABOUT]);
		//no cursor!!
		
	}
	
	//wait
	//own function
	WaitKey();
	
	//delete screen
	//own function
	DelScreen();
	
}


//calculate the new score
void CalcScore(void) {
	
	//variable
	int score = 0;
	char string[8];
	
	//calculate only if the human player of this various two player has won
	if (player[0].type == P_HUMAN && game.status == P0_WON) {
		//score = ((pot[F_REAL][7] - pot[F_REAL][14])*player[1].ai_strength/game.moves)*1000;
		//score-formula
		score = ((player[1].ai_strength*1000)/game.moves)*(pot[F_REAL][7] - pot[F_REAL][14])*(game.stones/3 + 1);
	}
	//the other possibility
	else if (game.status == P1_WON) {
		//score-formula
		score = ((player[0].ai_strength*1000)/game.moves)*(pot[F_REAL][14] - pot[F_REAL][7])*(game.stones/3 + 1);
	}
	
	//"copy" the score into a string
	//tigcclib: stdio.h, string.h
	sprintf(string, "%d", score);
	
	//draw the window
	//own function
	MenuDrawWindow(&menu[M_SCORE]);
	
	//draw the specific text
	//extgraph.h
	GrayDrawStrExt2B(0, menu[M_SCORE].y + 6, (char *)"Your Score", A_NORMAL | A_CENTERED, F_8x10, LPLANE, DPLANE);
	GrayDrawStrExt2B(104, menu[M_SCORE].y + 19, string, A_NORMAL, F_6x8, DPLANE, DPLANE);
	
	//wait and then delete the screen
	//own functions
	WaitKey();
	DelScreen();
	
	//update hiscores
	//changed function from www.technoplaza.net
	CalcHiscores(score);
	
}


//update the hiscores
//prototype from www.technoplaza.net / C-Programming/Graduate Review (Nibbler-game)
void CalcHiscores(unsigned int newscore) {
	
	//variables
	SCORE scores[MAX_HISCORES];
	int loop1;
	int pos = -1;
	char name[12];

	//look if there's a hiscore-file
	//changed function from www.technoplaza.net
	if (NeedHiscoreFile() == TRUE) {
		
		//if there isn't one, create one
		//changed function from www.technoplaza.net
		if (!CreateHiscoreFile()) {
			
			//error-handling
			//own function
			MenuMainRoutine(M_ERRORSAVE);
			
			//exit-value
			m_choice = -1;
			
		}
		
	}
	
	//load the hiscore-file
	//changed function from www.technoplaza.net
	if (!LoadHiscores(scores) && m_choice != -1) {
		
		//error-handling
		//own function
		MenuMainRoutine(M_ERRORLOAD);
		
		//exit-value
		m_choice = -1;
		
	}

	//check if latest score is a hiscore
	for (loop1 = (MAX_HISCORES - 1); loop1 >= 0 && m_choice != -1; loop1--) {
		
		//if it is
		if (newscore > scores[loop1].score) {
			
			//save the position
			pos = loop1;
			
		}
		
	}
	
	//if there is a new
	if (pos != -1) {
		
		//bring the hiscore-message
		//own function
		MenuMainRoutine(M_HISCORE);
		
		//draw the window for the name-input
		//own function
		MenuDrawWindow(&menu[M_NAME]);
		
		//get the name
		//own function
		InputStr(name, 8);
		
		//delete the screen
		//own function
		DelScreen();
		
		//update the whole hiscore-list (only if it's better than the last value
		//if (pos < (MAX_HISCORES - 1)) { //done before
			//move the necessary entries
			for (loop1 = (MAX_HISCORES - 1); loop1 > pos; loop1--) {
				
				//move the score
				scores[loop1].score = scores[loop1 - 1].score;
				//truncate the old name
				scores[loop1].name[0] = 0;
				//move the name
				strcpy(scores[loop1].name, scores[loop1 - 1].name);
				
			}
		//}

		// fill in the new hiscore
		scores[pos].score = newscore;
		scores[pos].name[0] = 0;
		strcpy(scores[pos].name, name);
		
		//save the table into the file
		//changed function from www.technoplaza.net
		if (!SaveHiscores(scores)) {
			
			//error handling
			//own function
			MenuMainRoutine(M_ERRORSAVE);
			
			//exit value
			m_choice = -1;
			
		}
		
	}
	else {
		
		//call the menu "no hiscore"
		//own function
		MenuMainRoutine(M_NOHISCORE);
		
	}
	
	// display the hiscore board
	if (m_choice != -1) {
		
		//call the function
		//own function
		DrawHiscores(scores);
			
	}
	
	
}


//=============================================================================
// file history
//=============================================================================
//
//	31.7.2004		movearrow()
//  13.8.2004		added the license
//	16.9.2004 	added all the menu-functions
//	9-12.10.04	added all the specific menu-functions
//	12.10.2004	added hiscore function
//	13.10.2004	bug fixes
