/******************************************************************************
*
* project name:    Kalaha
* file name:       Input21.c
* initial date:    15.09.2004
* author:          mhubi@freesurf.ch
* description:     input-routines
* 
* last changes:		 12.10.2004 12:00
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//include the main-header-file
#include "main-h21.h"

//extern variables used in this c-file
extern INT_HANDLER save_int_1;

//function which wait until no key is pressed (avoiding conflicts)
void Wait(void) {
	
	//tigcclib: kbd.h
	while (_rowread(0)) continue;
	
}

//function which waits until a key is pressed (no influence of/on anything else; only a stop)
void WaitKey(void) {
	
	int key = 0;
	
	//tigcclib: kbd.h
	while(_rowread(0)) continue;
	
	
	while (key != K_F2 && key != K_ENTER) {
		
		//own function
		key = KeyPressed();
		
		//set the calc in low-power-mode (wakes cpu only up for grayscale, keypress, amstimer, clock)
		//tigcclib: peekpoke.h
		pokeIO(0x600005, 0b10111);
		
	}
	
	//fastquit-possibility
	if (key == K_F2) {
		
		//set the status value
		game.status = FASTQUIT;
		
		//call the endblock of the program
		LeadOut();
		
	}
	
	//tigcclib: kbd.h
	while(_rowread(0)) continue;
	
}

//function which returns specific values, if a key is pressed or zero if there's none
int KeyPressed(void) {
	
	//tigcclib: kbd.h
	if (_keytest(RR_F2)) return K_F2;
	
	else if (_keytest(RR_ESC)) return K_ESC;
	
	else if (_keytest(RR_ENTER1) || _keytest(RR_ENTER2) || _keytest(RR_HAND)) return K_ENTER;
	
	else if (_keytest(RR_LEFT)) return K_LEFT;
	
	else if (_keytest(RR_RIGHT)) return K_RIGHT;
	
	else if (_keytest(RR_UP)) return K_UP;
	
	else if (_keytest(RR_DOWN)) return K_DOWN;
	
	else if (_keytest(RR_PLUS)) return K_PLUS;
	
	else if (_keytest(RR_MINUS)) return K_MINUS;
	
	return 0;
	
}


//function to insert a string with a limited length
//"own routine"; prototype from tigcc-documentation (faq: How can I get input from the keyboard?)
void InputStr(char *name, unsigned int maxlen) {
  
  //variables
  short key = 0;
  unsigned int position = 0;
  
  //set the font
  //tigcclib: graph.h
  FontSetSys(F_6x8);
  
  //temporarily reinstall the original auto_int_1 (--> use of ngetchx())
  //tigcclib: gray.h
  GraySetInt1Handler(save_int_1);

  
  //print "name" in black
  //extgraph.h
  GrayDrawStr(70, 60, "Name: ", A_NORMAL);
  
  //set the plane, so everything following is in dark-grey
  //tigcclib: gray.h
  GraySetAMSPlane(DARK_PLANE);
  
  //"clear" the buffer (truncating)
  name[0] = 0;
 	
 	//get input while the user doesn't press enter
 	while (key != KEY_ENTER) {
 		
 		//clear the place, where the name will appear
 		//tigcclib: graph.h
 		DrawStr(106, 60,"         ",A_SHADED);
  	//print the current name
  	DrawStr(106, 60, name, A_NORMAL);
  	//print cursor position
  	DrawStr(106 + 6*position, 60, "_", A_NORMAL);
  	
  	//get input
  	//tigcclib: kbd.h
  	key = ngetchx();
  	
  	//save the character and go to the next position
  	if (key >= ' ' && key <= '~' && position < maxlen) {
  		name[position++] = key;
   	}
   	
   	//delete the last character if backspace
  	else if (key == KEY_BACKSPACE && position != 0) {
			position--;
  	}
  	//fastquit possibility
  	else if (key == KEY_F2) {
  		
  		//set the status value
  		game.status = FASTQUIT;
  		
  		//call the endblock of the program
  		LeadOut();
  		
  	}
  	
  	//clear the next position
  	name[position] = 0;
	}
	
	//at the end: clear the cursor position
	//tigcclib: graph.h
  DrawStr(106 + 6*position, 60, " ", A_SHADED);
  
  //and set the auto_int_1 to the dummy handler
  //tigcclib: gray.h
  GraySetInt1Handler(DUMMY_HANDLER);
  
}


//=============================================================================
// file history
//=============================================================================
//
//	8.7.2004		wait() and waitkey() from the project scorch; keypressed() as
//							an attempt for a input-system started (as model the input-routines
//							of the tichess-program)
//	9.7.2004		more _keytest's in keypressed() (proved and reliable); deleted 
//							#includes
//	25.7.2004		added RR_HAND, WaitKey isn't out commented anymore
//  13.8.2004		added the license
//	16.9.2004		added RR_UP, RR_DOWN, RR_PLUS and RR_MINUS
//	12.10.2004	added InputStr()
//	 2.11.2004	added fastquit-possibility in waitkey()
