/******************************************************************************
*
* project name:    Kalaha
* file name:       Screen21.c
* initial date:    15.09.2004
* author:          mhubi@freesurf.ch
* description:     all graphic-routines
* 
* last changes:		 4.11.2004 15:00
*
*******************************************************************************
*
* Kalaha -- an board-game with alpha-beta-pruning-ai
* Copyright (C) 2004  Martin Huber
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
******************************************************************************/


//include the main-header-file
#include "main-h21.h"

//extern-variables, used in this c-file
extern unsigned int pot[8][15];
extern GAME game;
extern PLAYER player[2];


//deletes screen by fading out
void DelScreen(void) {
	
	//first invert the whole screen, and then let it fading out in a special way
	//extgraph.h
	GrayInvertRect(0, 0, 239, 127);
	FadeOutToWhite_LR18_R(LPLANE, DPLANE, 128, 30, 10);
	
}


//draws the whole game-field
void DrawEntireField(void) {
	
	unsigned int loop1;
	
	//clear both grayscale-planes
	//extgraph.h
	GrayClearScreen();
	
	//initialise and draw the background
	//own function
	DrawBackground();
	
	//draw all pots
	//own function
	DrawPots();
	
	//draw the stones in each pot
	//own function
	for (loop1 = 1; loop1 <= 14; loop1++) {
		DrawStones(loop1);
	}
	
	//draw the arrow
	//own function
	DrawArrow();
	
}


//initialise and draw the background
void DrawBackground(void) {
	
	unsigned int loop1;
	char string[12];
	
	//put the top-left part of the background
	//extgraph.h
	SpriteX8_OR_R(0, 0, 64, background1_4, 15, LPLANE);
	
	//generate and put the top-right part of the background
	//extgraph.h
	SpriteX8_MIRROR_H_R(64, background1_4, 15, background2_4);
	SpriteX8_OR_R(120, 0, 64, background2_4, 15, LPLANE);
	
	//generate and put the bottom-left part of the background
	//extgraph.h
	SpriteX8_MIRROR_V_R(64, background1_4, 15, background3_4);
	SpriteX8_OR_R(0, 63, 64, background3_4, 15, LPLANE);
	
	//generate and put the bottom-right part of the background
	//extgraph.h
	SpriteX8_MIRROR_H_R(64, background3_4, 15, background4_4);
	SpriteX8_OR_R(120, 63, 64, background4_4, 15, LPLANE);
	
	//draw the player-inscription
	
	//tigcclib: graph.h
	FontSetSys(F_8x10);
	
	//extgraph.h
	GrayDrawStr(1, 12, (char *)"P2", A_NORMAL);
	GrayDrawStr(222, 107, (char *)"P1", A_NORMAL);
	
	//tigcclib: graph.h
	FontSetSys(F_6x8);
	
	for (loop1 = 0; loop1 <= 1; loop1++) {
		
		if (player[loop1].type == P_HUMAN) {
			
			//extgraph.h
			GrayDrawStr(208 - loop1*207, 118 - loop1*117, (char *)"Human", A_NORMAL);
			
		}
		else {
			
			//extgraph.h
			GrayDrawStr(214 - loop1*213, 118 - loop1*117, (char *)"Calc", A_NORMAL);
			//tigcclib: stdio.h, string.h
			sprintf(string, "Level %d", player[loop1].ai_strength);
			//extgraph.h
			GrayDrawStr(loop1*196, 118 - loop1*117, string, A_NORMAL);
			
		}
		
	}
	
	#ifdef GRAPHICAL_VERSION
	//by compiling for the graphical-version, sprite the "kalaha" on the gamescreen
	//extgraph.h
	SpriteX8_OR_R(31, 50, 27, kalaha, 22, LPLANE);
	#endif
	
}


// deletes the corner of a rectangle in a specific plane
void DelCorners(void *plane, unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2) {
	
	//extgraph.h
	EXT_CLRPIX(plane, x1, y1);
	EXT_CLRPIX(plane, x1, y2);
	EXT_CLRPIX(plane, x2, y1);
	EXT_CLRPIX(plane, x2, y2);
	
}


//draw the pots
void DrawPots(void) {
	
	//loop-variables
	unsigned int loop1;
	
	for (loop1 = 1; loop1 <= 14; loop1++) {
		
		//if it's a big pot
		if (loop1%7 == 0) {
			
			//draw the big pots
			//extgraph.h
			GrayFastOutlineRect_R(LPLANE, DPLANE, lbcorner[loop1][0], lbcorner[loop1][1] - POT_HEIGHT2, lbcorner[loop1][0] + POT_WIDTH, lbcorner[loop1][1], COLOR_BLACK);
			
			//delete the corner of each box (--> light gray)
			//own function
			DelCorners(DPLANE, lbcorner[loop1][0], lbcorner[loop1][1] - POT_HEIGHT2, lbcorner[loop1][0] + POT_WIDTH, lbcorner[loop1][1]);
			
			//draw the area that must be reached to win
			//extgraph.h
			FastFillRect_R(LPLANE, lbcorner[loop1][0] + 1, lbcorner[loop1][1] - POT_HEIGHT2 + 1, lbcorner[loop1][0] + POT_WIDTH - 1, lbcorner[loop1][1] - 2 - game.stones*6, A_NORMAL);
			
			//make a light-gray frame inside the pot(temporarily?)
			//extgraph.h
			FastOutlineRect_R(LPLANE, lbcorner[loop1][0] + 1, lbcorner[loop1][1] - 38, lbcorner[loop1][0] + POT_WIDTH - 1, lbcorner[loop1][1] - 1, A_NORMAL);
						
			//put the sprite for the big-pot's inside on the screen
			//extgraph.h
			SpriteX8_OR(lbcorner[loop1][0] + 1, lbcorner[loop1][1] - 37, 36, big_pot, 3, LPLANE);
			
		}
		else {
			
			//draw the little pots
			//extgraph.h
			GrayFastOutlineRect_R(LPLANE, DPLANE, lbcorner[loop1][0], lbcorner[loop1][1] - POT_HEIGHT1, lbcorner[loop1][0] + POT_WIDTH, lbcorner[loop1][1], COLOR_BLACK);
			
			//delete the corner of each box (--> light gray)
			//own function
			DelCorners(DPLANE, lbcorner[loop1][0], lbcorner[loop1][1] - POT_HEIGHT1, lbcorner[loop1][0] + POT_WIDTH, lbcorner[loop1][1]);
			
			//put the sprite for the little-pot's inside on the screen(old version without background)
			//extgraph.h
			SpriteX8_OR(lbcorner[loop1][0] + 1, lbcorner[loop1][1] - POT_HEIGHT1 + 2, 29, wee_pot, 3, LPLANE);
			
			//make a light-gray frame inside the pot(temporarily)
			//extgraph.h
			FastOutlineRect_R(LPLANE, lbcorner[loop1][0] + 1, lbcorner[loop1][1] - POT_HEIGHT1 + 1, lbcorner[loop1][0] + POT_WIDTH - 1, lbcorner[loop1][1] - 1, A_NORMAL);
			
		}
		
	}
	

}


//clears the content of a pot
void ClearPot(unsigned int potnum) {
	
	//clear the pot (without the light-gray frame)
	//extgraph.h
	GrayFastFillRect_R(LPLANE, DPLANE, lbcorner[potnum][0] + 2, lbcorner[potnum][1] - POT_HEIGHT1 + 2, lbcorner[potnum][0] + POT_WIDTH - 2, lbcorner[potnum][1] - 2, COLOR_WHITE);
	
	//put the sprite again on the screen
	//extgraph.h
	SpriteX8_OR(lbcorner[potnum][0] + 1, lbcorner[potnum][1] - POT_HEIGHT1 + 2, 29, wee_pot, 3, LPLANE);
	
}


// draws the arrow at the current position saved in game.cur_pos
void DrawArrow(void) {
	
	unsigned int x, y;
	
	//calc the coordinates
	x = lbcorner[game.cur_pos][0] + POT_WIDTH/2 - 3;
	y = lbcorner[game.cur_pos][1] + 2;
	
	//first save the background, where the arrow will be drawn
	//extgraph.h
	Sprite8Get_R(x, y, 8, LPLANE, arrow_mask1);

	//draw the arrow
	//extgraph.h
	GraySprite8_OR_R(x, y, 8, arrow1, arrow2, LPLANE, DPLANE);
	
}


// delete the arrow at the current position saved in game.cur_pos
void DelArrow(void) {
	
	unsigned int x1, y1;
	
	//calculate the coordinates
	x1 = lbcorner[game.cur_pos][0] + POT_WIDTH/2 - 3;
	y1 = lbcorner[game.cur_pos][1] + 2;
	
	//delete the arrow by masking out the dark-plane and put the saved background-part back on the screen
	//extgraph.h
	GraySprite8_AND_R(x1, y1, 8, arrow_mask1, arrow_mask2, LPLANE, DPLANE);
	
}


// draws all stones of a pot into the pot
void DrawStones(unsigned int potnum) {
	
	//variable
	unsigned int loop1, x, y;
	
	//avoid an overflow of the graphical pots
	if (pot[F_REAL][potnum] > 30 && potnum%7 != 0) {
		
		//set the fastquit-value
		game.status = FASTQUIT;
		
		//end the game
		//own function
		LeadOut();
		
	}
	
	// for-loop --> each stone is drawn (drawing in the real field)
	else for (loop1 = 0; loop1 < pot[F_REAL][potnum]; loop1++) {
		
		//calculate coordinates
		x = lbcorner[potnum][0] + 2 + (loop1%6)*4;
		y = lbcorner[potnum][1] - 6 - ((int)(loop1/6))*6;
		
		//this is only possible in the big pots, and then you've won, so i don't mask it out anymore
		//so you see when you reached the necessary number of stones
		if (loop1 < 36) {
			
			//extgraph.h
			Sprite8_AND_R(x, y, 6, stone_mask, LPLANE);
			
		}
				
		//"or" the sprite to the dark planes
		//extgraph.h
		Sprite8_OR_R(x, y, 6, stone, DPLANE);
		
	} 
	
	
}


//draws the hiscore-table
void DrawHiscores(SCORE *scores) {
	
	//variables
	unsigned int loop1;
	char string[8];
	
	//draw the window
	//own function
	MenuDrawWindow(&menu[M_HIGHTABLE]);
	
	//draw the title
	//extgraph.h
	GrayDrawStrExt2B(0, menu[M_HIGHTABLE].y + 6, (char *)"HIGHSCORES", A_NORMAL | A_CENTERED, F_8x10, LPLANE, DPLANE);
	
	//change the font
	//tigcclib: graph.h
	FontSetSys(F_6x8);
	
	//loop through the five entries
	for (loop1 = 0; loop1 < 5; loop1++) {
		
		//print the rank
		//tigcclib: stdio.h, string.h
		sprintf(string, "%d.", loop1 + 1);
		//extgraph.h
		GrayDrawStr2B(58, menu[M_HIGHTABLE].y + 19 + loop1*9, string, A_NORMAL, DPLANE, DPLANE);
		//print the name
		//extgraph.h
		GrayDrawStr2B(70, menu[M_HIGHTABLE].y + 19 + loop1*9, scores[loop1].name, A_NORMAL, DPLANE, DPLANE);
		//print the score
		//tigcclib: stdio.h, string.h
		sprintf(string, "%d", scores[loop1].score);
		//extgraph.h
		GrayDrawStr2B(150, menu[M_HIGHTABLE].y + 19 + loop1*9, string, A_NORMAL, DPLANE, DPLANE);
		
	}
	
	//wait on user-key
	//own function
	WaitKey();
	
	//delete screen
	//own function
	DelScreen();
	
}

#ifndef GRAPHICAL_VERSION
//compile also these functions for ai-message drawing (only in the non-graphical-version)
//delete the content of the message-area
void MsgDelMiddle(void) {
	
	//draw a filled, white rectangle
	//extgraph.h
	GrayFastFillRect_R(LPLANE, DPLANE, 29, 48, 211, 78, COLOR_WHITE);
	
}


//draws the changes of the search-depth (special cases)
void MsgDrawAidepth(void) {
	
	char string[20];
	
	//delete the area and print the number
	//tigcclib: graph.h
	FontSetSys(F_4x6);
	//extgraph.h
	GrayDrawStr(181,64,"          ",A_SHADED);
	//tigcclib: stdio.h, string.h
	sprintf(string, "depth    : %u", ai.max_depth);
	//extgraph.h
	GrayDrawStr(146,64,string,A_NORMAL);
	
}


//draws the progress of the ai (each 1000 endnode)
void MsgDrawAiNodes(void) {
	
	char string[20];
	
	//delete and print the area of the number
	//tigcclib: graph.h
	FontSetSys(F_4x6);
	//extgraph.h
	GrayDrawStr(181,58,"          ",A_SHADED);
	//tigcclib: stdio.h, string.h
	sprintf(string, "progress: %u", ai.end_node);
	//extgraph.h
	GrayDrawStr(146,58,string,A_NORMAL);
	
}


//draw the important values of the ai into the message-area
void MsgDrawAiEnd(int value) {
	
	char string[20];
	
	//tigcclib: graph.h
	FontSetSys(F_4x6);
	
	//print the value of the move
	//tigcclib: stdio.h, string.h
	sprintf (string, "value: %d", value);
	//extgraph.h
	GrayDrawStr(32, 58, string, A_NORMAL);
	
	//print the chosen pot-number
	//tigcclib: stdio.h, string.h
	sprintf (string, "pot    : %d", ai.end_move);
	//extgraph.h
	GrayDrawStr(32, 64, string, A_NORMAL);
	
	//print the number of endnodes
	//tigcclib: stdio.h, string.h
	sprintf (string, "endnodes  : %d", ai.end_node);
	//extgraph.h
	GrayDrawStr(80, 58, string, A_NORMAL);
	
	//print the number of evaluated endnodes
	//tigcclib: stdio.h, string.h
	sprintf (string, "evaluation: %d", ai.evaluate);
	//extgraph.h
	GrayDrawStr(80, 64, string, A_NORMAL);
	
}
#endif


//=============================================================================
// file history
//=============================================================================
//
//   5.7.2004	drawpots(), drawarrow()
//   6.7.2004	optimisation of drawpots() --> delcorners() (new)
//   8.7.2004	integration in the multi-file-project --> no main-function anymore
//						drawstones(), new version of drawpots() and drawarrow() so drawstones()
//  					, drawpots() and drawarrow() became easy/ier (-->lbcorner)
//   9.7.2004	clearpot(), delarrow(); deleted #include's
//  12.7.2004	optimations of drawarrow(), delarrow() and drawstones(): better to
//						understand and lower file-size of the .v2z; added new externs
//						and new versions of the functions that use them; drawpots(): the big
//						pots have now a mark until where you must fill them to win; changes in
//						drawarrow();
//  24.7.2004	added drawresult()
//   2.8.2004	changed the pot --> 2nd index
//  13.8.2004	added the license
//  19.8.2004	added the sprite-routine for the pot-inside and made the adjustments
//  31.8.2004	added the first background-routine, made the adjustments in the arrow-routines
//   2.9.2004	added the new background-routine
//  16.9.2004	added msgdelmiddle(), msgdrawaiend(), msgdrawainodes()
// 10.10.2004	changed the player inscription in drawbackground()
// 11.10.2004	changed drawpots() --> variable number of stones
// 12.10.2004	deleted drawresult(), added delscreen() and drawhiscores()
// 13.10.2004	added msgdrawaidepth()
// 21.10.2004	added the level-printing
//  4.11.2004	added #ifdef + #ifndef and sprite-function for "kalaha" in drawbackground()
